#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup


setup(name='qree',
      version='0.0.3',
      description='Qree: Tiny but mighty Python templating.',
      author='Polydojo, Inc.',
      author_email='8079593+sumukhbarve@users.noreply.github.com',
      url='https://github.com/polydojo/qree',
      py_modules=['qree'],
     )
